class Pool
  def initialize(klass, init_func, valid_func, destroy_func=nil)
    @klass = klass
    raise "Error: '#{init_func.inspect}' not a proc" unless init_func.respond_to?(:call)
    @init_func = init_func
    raise "Error" unless valid_func.is_a? Proc
    @valid_func = valid_func
    raise "Error" unless destroy_func.nil? || destroy_func.is_a?(Proc)
    @destroy_func = destroy_func
    @pool = []
  end
  
  def get(*args)
    (@pool.size>0 ? @pool.shift : @klass.new).tap{|o|
      @init_func.call(o, *args, &(block_given? ? Proc.new : nil)) }
  end
  
  def release(obj)
    if @valid_func.call(obj) then
      @pool << obj
    else
      @destroy_func.call(obj) unless @destroy_func.nil?
    end
  end
  
end